import PQueue from "p-queue";
import { Log, ISortFunction, TrimOptions, LogOptions, Identity, CanAppend, Change, TrimToLengthOption, TrimToByteLengthOption } from "@dao-xyz/peerbit-log";
import { Encoding, EncryptionTemplateMaybeEncrypted } from "@dao-xyz/peerbit-log";
import { Entry } from "@dao-xyz/peerbit-log";
import { BlockStore } from "@dao-xyz/libp2p-direct-block";
import Cache from "@dao-xyz/peerbit-cache";
import { Constructor } from "@dao-xyz/borsh";
import { PublicKeyEncryptionResolver } from "@dao-xyz/peerbit-crypto";
import { EntryWithRefs } from "./entry-with-refs.js";
export declare class CachedValue {
}
export type AddOperationOptions<T> = {
    skipCanAppendCheck?: boolean;
    identity?: Identity;
    nexts?: Entry<T>[];
    trim?: TrimOptions | TrimToByteLengthOption | TrimToLengthOption;
    reciever?: EncryptionTemplateMaybeEncrypted;
};
export declare class CID extends CachedValue {
    hash: string;
    constructor(opts?: {
        hash: string;
    });
}
export declare class UnsfinishedReplication extends CachedValue {
    hashes: string[];
    constructor(opts?: {
        hashes: string[];
    });
}
export declare class HeadsCache<T> extends CachedValue {
    heads: string[];
    last?: string;
    counter: bigint;
    constructor(heads: string[], counter: bigint, last?: string);
}
export interface IStoreOptions<T> {
    /**
     * f set to true, will throw an error if the database can't be found locally. (Default: false)
     */
    localOnly?: boolean;
    /**
     * The directory where data will be stored
     */
    directory?: string;
    onClose?: (store: Store<T>) => void;
    onDrop?: (store: Store<T>) => void;
    onLoad?: (store: Store<T>) => void;
    onLoadProgress?: (store: Store<T>, entry: Entry<T>) => void;
    onWrite?: (store: Store<T>, _entry: Entry<T>) => void;
    onOpen?: (store: Store<any>) => Promise<void>;
    onReplicationQueued?: (store: Store<any>, entry: Entry<T>) => void;
    onReplicationFetch?: (store: Store<any>, entry: Entry<T>) => void;
    onReplicationComplete?: (store: Store<any>) => void;
    onReady?: (store: Store<T>) => void;
    saveFile?: (file: any) => Promise<string>;
    loadFile?: (cid: string) => Promise<Uint8Array>;
    encryption?: PublicKeyEncryptionResolver;
    maxHistory?: number;
    fetchEntryTimeout?: number;
    replicationConcurrency?: number;
    sortFn?: ISortFunction;
    trim?: TrimOptions;
    onUpdate?: (change: Change<T>) => void;
}
export interface IInitializationOptions<T> extends IStoreOptions<T>, IInitializationOptionsDefault<T> {
    resolveCache: (store: Store<any>) => Promise<Cache<CachedValue>> | Cache<CachedValue>;
}
interface IInitializationOptionsDefault<T> {
    maxHistory?: number;
    replicationConcurrency?: number;
    typeMap?: {
        [key: string]: Constructor<any>;
    };
    cacheId: string;
}
export declare const DefaultOptions: IInitializationOptionsDefault<any>;
export interface Initiable<T> {
    init?(blockstore: BlockStore, identity: Identity, options: IInitializationOptions<T>): Promise<this>;
}
export declare class Store<T> implements Initiable<T> {
    _storeIndex: number;
    _canAppend?: CanAppend<T>;
    _onUpdate?: (change: Change<T>) => Promise<void> | void;
    _onUpdateOption?: (change: Change<T>) => Promise<void> | void;
    _options: IInitializationOptions<T>;
    identity: Identity;
    _headsPathCounter: number;
    headsPath: string;
    _lastHeadsPath?: string;
    _lastHeadsCount: bigint;
    removedHeadsPath: string;
    _lastRemovedHeadsPath?: string;
    _lastRemovedHeadsCount: bigint;
    _asyncJobs: Promise<void[] | void>;
    snapshotPath: string;
    initialized: boolean;
    encoding: Encoding<T>;
    _store: BlockStore;
    _cache: Cache<CachedValue>;
    _oplog: Log<T>;
    _queue: PQueue<any, any>;
    _txsQueue: PQueue<any, any>;
    _key: string;
    _saveFile: (file: any) => Promise<string>;
    _loadFile: (cid: string) => Promise<Uint8Array | undefined>;
    constructor(properties?: {
        storeIndex: number;
    });
    setup(properties: {
        encoding: Encoding<T>;
        canAppend: CanAppend<T>;
        onUpdate: (change: Change<T>) => void;
    }): void;
    init(store: BlockStore, identity: Identity, options: IInitializationOptions<T>): Promise<this>;
    updateCachedHeads(change: {
        added: (Entry<T> | string)[];
        removed: (Entry<T> | string)[];
    }, reset?: boolean): Promise<void>;
    idle(): Promise<any>;
    getCachedHeads(lastHeadsPath?: string | undefined, lastRemovedHeadsPath?: string | undefined): Promise<string[]>;
    get id(): string;
    get oplog(): Log<any>;
    get key(): string;
    get logOptions(): LogOptions<T>;
    setIdentity(identity: Identity): void;
    set canAppend(canAppend: CanAppend<T> | undefined);
    get canAppend(): CanAppend<T> | undefined;
    set onUpdate(onUpdate: (change: Change<T>) => void);
    get closed(): boolean;
    close(): Promise<void>;
    /**
     * Drops a database and removes local data
     */
    drop(): Promise<void>;
    load(amount?: number, opts?: {
        fetchEntryTimeout?: number;
    }): Promise<void>;
    loadLastHeadsPath(): Promise<void>;
    getCachedHeadsCount(headPath?: string): Promise<bigint>;
    addOperation(data: T, options?: AddOperationOptions<T>): Promise<{
        entry: Entry<T>;
        removed: Entry<T>[];
    }>;
    removeOperation(entry: Entry<T> | Entry<T>[], options?: {
        recursively?: boolean;
    }): Promise<Change<T>>;
    /**
     *
     * @param entries
     * @returns change
     */
    sync(entries: (EntryWithRefs<T> | Entry<T> | string)[], options?: {
        canAppend?: CanAppend<T>;
        save: boolean;
    }): Promise<boolean>;
    saveSnapshot(): Promise<string[]>;
    loadFromSnapshot(): Promise<this>;
    _updateIndex(change: Change<T>): Promise<void>;
    _onLoadProgress(entry: Entry<any>): void;
    clone(): Store<T>;
}
export {};
